@@ -23,7 +23,9 @@
  * Author:  Keith Packard, SuSE, Inc.
  */
 
+#ifdef HAVE_CONFIG_H
 #include <config.h>
+#endif
 #include <string.h>
 #include "pixman-private.h"
 #include "pixman-combine32.h"
@@ -147,7 +149,9 @@ fast_composite_over_x888_8_8888 (pixman_implementation_t *imp,
 		s = *src | 0xff000000;
 
 		if (m == 0xff)
+		{
 		    *dst = s;
+		}
 		else
 		{
 		    d = in (s, m);
@@ -202,14 +206,12 @@ fast_composite_in_n_8_8 (pixman_implementation_t *imp,
 	    while (w--)
 	    {
 		m = *mask++;
+
 		if (m == 0)
-		{
 		    *dst = 0;
-		}
 		else if (m != 0xff)
-		{
 		    *dst = MUL_UN8 (m, *dst, t);
-		}
+
 		dst++;
 	    }
 	}
@@ -228,14 +230,12 @@ fast_composite_in_n_8_8 (pixman_implementation_t *imp,
 	    {
 		m = *mask++;
 		m = MUL_UN8 (m, srca, t);
+
 		if (m == 0)
-		{
 		    *dst = 0;
-		}
 		else if (m != 0xff)
-		{
 		    *dst = MUL_UN8 (m, *dst, t);
-		}
+
 		dst++;
 	    }
 	}
@@ -278,14 +278,12 @@ fast_composite_in_8_8 (pixman_implementation_t *imp,
 	while (w--)
 	{
 	    s = *src++;
+
 	    if (s == 0)
-	    {
 		*dst = 0;
-	    }
 	    else if (s != 0xff)
-	    {
 		*dst = MUL_UN8 (s, *dst, t);
-	    }
+
 	    dst++;
 	}
     }
@@ -349,6 +347,67 @@ fast_composite_over_n_8_8888 (pixman_implementation_t *imp,
     }
 }
 
+static void
+fast_composite_add_n_8888_8888_ca (pixman_implementation_t *imp,
+				   pixman_op_t              op,
+				   pixman_image_t *         src_image,
+				   pixman_image_t *         mask_image,
+				   pixman_image_t *         dst_image,
+				   int32_t                  src_x,
+				   int32_t                  src_y,
+				   int32_t                  mask_x,
+				   int32_t                  mask_y,
+				   int32_t                  dest_x,
+				   int32_t                  dest_y,
+				   int32_t                  width,
+				   int32_t                  height)
+{
+    uint32_t src, srca, s;
+    uint32_t    *dst_line, *dst, d;
+    uint32_t    *mask_line, *mask, ma;
+    int dst_stride, mask_stride;
+    uint16_t w;
+
+    src = _pixman_image_get_solid (src_image, dst_image->bits.format);
+
+    srca = src >> 24;
+    if (src == 0)
+	return;
+
+    PIXMAN_IMAGE_GET_LINE (dst_image, dest_x, dest_y, uint32_t, dst_stride, dst_line, 1);
+    PIXMAN_IMAGE_GET_LINE (mask_image, mask_x, mask_y, uint32_t, mask_stride, mask_line, 1);
+
+    while (height--)
+    {
+	dst = dst_line;
+	dst_line += dst_stride;
+	mask = mask_line;
+	mask_line += mask_stride;
+	w = width;
+
+	while (w--)
+	{
+	    ma = *mask++;
+
+	    if (ma == 0xffffffff && srca == 0xff)
+	    {
+		*dst = src;
+	    }
+	    else if (ma)
+	    {
+		d = *dst;
+		s = src;
+		
+		UN8x4_MUL_UN8x4_ADD_UN8x4 (s, ma, d);
+
+		*dst = s;
+	    }
+
+	    dst++;
+	}
+    }
+}
+
 static void
 fast_composite_over_n_8888_8888_ca (pixman_implementation_t *imp,
                                     pixman_op_t              op,
@@ -364,7 +423,7 @@ fast_composite_over_n_8888_8888_ca (pixman_implementation_t *imp,
                                     int32_t                  width,
                                     int32_t                  height)
 {
-    uint32_t src, srca;
+    uint32_t src, srca, s;
     uint32_t    *dst_line, *dst, d;
     uint32_t    *mask_line, *mask, ma;
     int dst_stride, mask_stride;
@@ -400,11 +459,12 @@ fast_composite_over_n_8888_8888_ca (pixman_implementation_t *imp,
 	    else if (ma)
 	    {
 		d = *dst;
+		s = src;
 
-		UN8x4_MUL_UN8x4 (src, ma);
+		UN8x4_MUL_UN8x4 (s, ma);
 		UN8x4_MUL_UN8 (ma, srca);
 		ma = ~ma;
-		UN8x4_MUL_UN8x4_ADD_UN8x4 (d, ma, src);
+		UN8x4_MUL_UN8x4_ADD_UN8x4 (d, ma, s);
 
 		*dst = d;
 	    }
@@ -459,7 +519,9 @@ fast_composite_over_n_8_0888 (pixman_implementation_t *imp,
 	    if (m == 0xff)
 	    {
 		if (srca == 0xff)
+		{
 		    d = src;
+		}
 		else
 		{
 		    d = fetch_24 (dst);
@@ -522,7 +584,9 @@ fast_composite_over_n_8_0565 (pixman_implementation_t *imp,
 	    if (m == 0xff)
 	    {
 		if (srca == 0xff)
+		{
 		    d = src;
+		}
 		else
 		{
 		    d = *dst;
@@ -556,11 +620,11 @@ fast_composite_over_n_8888_0565_ca (pixman_implementation_t *imp,
                                     int32_t                  width,
                                     int32_t                  height)
 {
-    uint32_t src, srca;
-    uint16_t src16;
-    uint16_t    *dst_line, *dst;
-    uint32_t d;
-    uint32_t    *mask_line, *mask, ma;
+    uint32_t  src, srca, s;
+    uint16_t  src16;
+    uint16_t *dst_line, *dst;
+    uint32_t  d;
+    uint32_t *mask_line, *mask, ma;
     int dst_stride, mask_stride;
     uint16_t w;
 
@@ -604,10 +668,12 @@ fast_composite_over_n_8888_0565_ca (pixman_implementation_t *imp,
 		d = *dst;
 		d = CONVERT_0565_TO_0888 (d);
 
-		UN8x4_MUL_UN8x4 (src, ma);
+		s = src;
+
+		UN8x4_MUL_UN8x4 (s, ma);
 		UN8x4_MUL_UN8 (ma, srca);
 		ma = ~ma;
-		UN8x4_MUL_UN8x4_ADD_UN8x4 (d, ma, src);
+		UN8x4_MUL_UN8x4_ADD_UN8x4 (d, ma, s);
 
 		*dst = CONVERT_8888_TO_0565 (d);
 	    }
@@ -662,19 +728,19 @@ fast_composite_over_8888_8888 (pixman_implementation_t *imp,
 }
 
 static void
-fast_composite_src_8888_0888 (pixman_implementation_t *imp,
-                              pixman_op_t              op,
-                              pixman_image_t *         src_image,
-                              pixman_image_t *         mask_image,
-                              pixman_image_t *         dst_image,
-                              int32_t                  src_x,
-                              int32_t                  src_y,
-                              int32_t                  mask_x,
-                              int32_t                  mask_y,
-                              int32_t                  dest_x,
-                              int32_t                  dest_y,
-                              int32_t                  width,
-                              int32_t                  height)
+fast_composite_over_8888_0888 (pixman_implementation_t *imp,
+			       pixman_op_t              op,
+			       pixman_image_t *         src_image,
+			       pixman_image_t *         mask_image,
+			       pixman_image_t *         dst_image,
+			       int32_t                  src_x,
+			       int32_t                  src_y,
+			       int32_t                  mask_x,
+			       int32_t                  mask_y,
+			       int32_t                  dest_x,
+			       int32_t                  dest_y,
+			       int32_t                  width,
+			       int32_t                  height)
 {
     uint8_t     *dst_line, *dst;
     uint32_t d;
@@ -752,7 +818,9 @@ fast_composite_over_8888_0565 (pixman_implementation_t *imp,
 	    if (s)
 	    {
 		if (a == 0xff)
+		{
 		    d = s;
+		}
 		else
 		{
 		    d = *dst;
@@ -983,10 +1051,14 @@ fast_composite_solid_fill (pixman_implementation_t *imp,
     src = _pixman_image_get_solid (src_image, dst_image->bits.format);
 
     if (dst_image->bits.format == PIXMAN_a8)
+    {
 	src = src >> 24;
+    }
     else if (dst_image->bits.format == PIXMAN_r5g6b5 ||
              dst_image->bits.format == PIXMAN_b5g6r5)
+    {
 	src = CONVERT_8888_TO_0565 (src);
+    }
 
     pixman_fill (dst_image->bits.bits, dst_image->bits.rowstride,
                  PIXMAN_FORMAT_BPP (dst_image->bits.format),
@@ -1056,6 +1128,7 @@ static const pixman_fast_path_t c_fast_paths[] =
     { PIXMAN_OP_ADD, PIXMAN_a8r8g8b8,  PIXMAN_null,     PIXMAN_a8r8g8b8, fast_composite_add_8888_8888,   0 },
     { PIXMAN_OP_ADD, PIXMAN_a8b8g8r8,  PIXMAN_null,     PIXMAN_a8b8g8r8, fast_composite_add_8888_8888,   0 },
     { PIXMAN_OP_ADD, PIXMAN_a8,        PIXMAN_null,     PIXMAN_a8,       fast_composite_add_8000_8000,   0 },
+    { PIXMAN_OP_ADD, PIXMAN_solid,     PIXMAN_a8r8g8b8, PIXMAN_a8r8g8b8, fast_composite_add_n_8888_8888_ca, NEED_COMPONENT_ALPHA },
     { PIXMAN_OP_ADD, PIXMAN_solid,     PIXMAN_a8,       PIXMAN_a8,       fast_composite_add_8888_8_8,    0 },
     { PIXMAN_OP_SRC, PIXMAN_solid,     PIXMAN_null,     PIXMAN_a8r8g8b8, fast_composite_solid_fill, 0 },
     { PIXMAN_OP_SRC, PIXMAN_solid,     PIXMAN_null,     PIXMAN_x8r8g8b8, fast_composite_solid_fill, 0 },
@@ -1211,8 +1284,8 @@ fast_path_composite (pixman_implementation_t *imp,
         && (src->common.filter == PIXMAN_FILTER_NEAREST)
         && PIXMAN_FORMAT_BPP (dest->bits.format) == 32
         && src->bits.format == dest->bits.format
-        && !src->common.read_func && !src->common.write_func
-        && !dest->common.read_func && !dest->common.write_func)
+        && !src->bits.read_func && !src->bits.write_func
+        && !dest->bits.read_func && !dest->bits.write_func)
     {
 	/* ensure that the transform matrix only has a scale */
 	if (src->common.transform->matrix[0][1] == 0 &&
