@@ -63,7 +63,47 @@
 #define PIXREGION_TOP(reg) PIXREGION_BOX (reg, (reg)->data->numRects)
 #define PIXREGION_END(reg) PIXREGION_BOX (reg, (reg)->data->numRects - 1)
 
-#define GOOD(reg) assert (PREFIX (_selfcheck) (reg))
+#define GOOD_RECT(rect) ((rect)->x1 < (rect)->x2 && (rect)->y1 < (rect)->y2)
+#define BAD_RECT(rect) ((rect)->x1 > (rect)->x2 || (rect)->y1 > (rect)->y2)
+
+#define noPIXMAN_REGION_LOG_FAILURES
+
+#if defined PIXMAN_REGION_LOG_FAILURES || defined PIXMAN_REGION_DEBUG
+
+static void
+log_region_error (const char *function, const char *message)
+{
+    static int n_messages = 0;
+
+    if (n_messages < 50)
+    {
+	fprintf (stderr,
+		 "*** BUG ***\n"
+		 "%s: %s\n"
+		 "Set a breakpoint on 'log_region_error' to debug\n\n",
+                 function, message);
+
+#if defined PIXMAN_REGION_DEBUG
+        abort ();
+#endif
+
+	n_messages++;
+    }
+}
+
+#define GOOD(reg)							\
+    do									\
+    {									\
+	if (!PREFIX (_selfcheck (reg)))					\
+	    log_region_error (FUNC, "Malformed region " # reg);         \
+    } while (0)
+
+#else
+
+#define log_region_error(function, name)
+#define GOOD(reg)
+
+#endif
 
 static const box_type_t PREFIX (_empty_box_) = { 0, 0, 0, 0 };
 static const region_data_type_t PREFIX (_empty_data_) = { 0, 0 };
@@ -344,12 +384,27 @@ PREFIX (_init_rect) (region_type_t *	region,
     region->extents.x2 = x + width;
     region->extents.y2 = y + height;
 
+    if (!GOOD_RECT (&region->extents))
+    {
+        if (BAD_RECT (&region->extents))
+            log_region_error (FUNC, "Invalid rectangle passed");
+        PREFIX (_init) (region);
+        return;
+    }
+
     region->data = NULL;
 }
 
 PIXMAN_EXPORT void
 PREFIX (_init_with_extents) (region_type_t *region, box_type_t *extents)
 {
+    if (!GOOD_RECT (extents))
+    {
+        if (BAD_RECT (extents))
+            log_region_error (FUNC, "Invalid rectangle passed");
+        PREFIX (_init) (region);
+        return;
+    }
     region->extents = *extents;
 
     region->data = NULL;
@@ -455,7 +510,7 @@ PREFIX (_copy) (region_type_t *dst, region_type_t *src)
 {
     GOOD (dst);
     GOOD (src);
-    
+
     if (dst == src)
 	return TRUE;
     
@@ -1292,14 +1347,18 @@ PREFIX (_union_rect) (region_type_t *dest,
 {
     region_type_t region;
 
-    if (!width || !height)
-	return PREFIX (_copy) (dest, source);
-    
     region.extents.x1 = x;
     region.extents.y1 = y;
     region.extents.x2 = x + width;
     region.extents.y2 = y + height;
 
+    if (!GOOD_RECT (&region.extents))
+    {
+        if (BAD_RECT (&region.extents))
+            log_region_error (FUNC, "Invalid rectangle passed");
+	return PREFIX (_copy) (dest, source);
+    }
+    
     region.data = NULL;
 
     return PREFIX (_union) (dest, source, &region);
@@ -2229,6 +2288,8 @@ PREFIX (_translate) (region_type_t *region, int x, int y)
 	    }
 	}
     }
+
+    GOOD (region);
 }
 
 PIXMAN_EXPORT void
@@ -2236,14 +2297,13 @@ PREFIX (_reset) (region_type_t *region, box_type_t *box)
 {
     GOOD (region);
 
-    assert (box->x1 <= box->x2);
-    assert (box->y1 <= box->y2);
+    assert (GOOD_RECT (box));
 
     region->extents = *box;
 
     FREE_DATA (region);
 
-    region->data = (region_data_type_t *)NULL;
+    region->data = NULL;
 }
 
 /* box is "return" value */
