@@ -80,7 +80,7 @@ clip_general_image (pixman_region32_t * region,
 
 static inline pixman_bool_t
 clip_source_image (pixman_region32_t * region,
-                   pixman_image_t *    picture,
+                   pixman_image_t *    image,
                    int                 dx,
                    int                 dy)
 {
@@ -89,11 +89,11 @@ clip_source_image (pixman_region32_t * region,
      * the clip was not set by a client, then it is a hierarchy
      * clip and those should always be ignored for sources).
      */
-    if (!picture->common.clip_sources || !picture->common.client_clip)
+    if (!image->common.clip_sources || !image->common.client_clip)
 	return TRUE;
 
     return clip_general_image (region,
-                               &picture->common.clip_region,
+                               &image->common.clip_region,
                                dx, dy);
 }
 
@@ -663,7 +663,8 @@ _pixman_run_fast_path (const pixman_fast_path_t *paths,
 	if (has_fast_path && src->type == BITS)
 	{
 	    has_fast_path = !src->bits.read_func &&
-	                    !src->bits.write_func;
+	                    !src->bits.write_func &&
+		            !PIXMAN_FORMAT_IS_WIDE (src->bits.format);
 	}
     }
 
@@ -674,9 +675,10 @@ _pixman_run_fast_path (const pixman_fast_path_t *paths,
 	                !mask->common.alpha_map &&
 	                !mask->bits.read_func &&
 	                !mask->bits.write_func &&
-			mask->common.filter != PIXMAN_FILTER_CONVOLUTION &&
-			mask->common.repeat != PIXMAN_REPEAT_PAD &&
-			mask->common.repeat != PIXMAN_REPEAT_REFLECT;
+			 mask->common.filter != PIXMAN_FILTER_CONVOLUTION &&
+			 mask->common.repeat != PIXMAN_REPEAT_PAD &&
+			 mask->common.repeat != PIXMAN_REPEAT_REFLECT;
+	                !PIXMAN_FORMAT_IS_WIDE (src->bits.format);
     }
 
     if (has_fast_path)
